defmodule Clickhouse.Format do
  @moduledoc """
  A behaviour to implement data formatting of Clickhouse queries and results.
  """

  @type t :: module()

  @callback decode(Clickhouse.Result.t()) ::
              {:ok, Clickhouse.Result.t()} | {:error, Clickhouse.Error.t()}

  @callback encode(Clickhouse.Query.t()) ::
              {:ok, Clickhouse.Query.t()} | {:error, Clickhouse.Error.t()}

  alias Clickhouse.Error

  @formats %{
    Clickhouse.Format.TSV => [
      "TabSeparated",
      "TSV"
    ]
  }

  @spec find(binary) :: {:ok, Clickhouse.Format.t()} | {:error, Clickhouse.Error.t()}
  for {module, names} <- @formats do
    for name <- names do
      def find(unquote(name)), do: {:ok, unquote(module)}
    end
  end

  def find(format) do
    {:error, Error.new(:not_implemented, "Format #{format} not implemented.")}
  end

  @spec decode(Clickhouse.Result.t()) ::
          {:ok, Clickhouse.Result.t()} | {:error, Clickhouse.Error.t()}
  def decode(%{__format__: nil} = result), do: {:ok, result}
  def decode(result), do: result.__format__.decode(result)

  @spec encode(Clickhouse.Query.t()) ::
          {:ok, Clickhouse.Query.t()} | {:error, Clickhouse.Error.t()}
  def encode(%{__format__: nil} = query), do: {:ok, query}
  def encode(query), do: query.__format__.encode(query)
end
