defmodule Clickhouse.Interface do
  @moduledoc """
  A behaviour to implement an interface to interact with a Clickhouse server.
  """

  @callback start_link(keyword()) :: GenServer.on_start()
  @callback execute(Clickhouse.Query.t()) ::
              {:ok, Clickhouse.Result.t()} | {:error, Clickhouse.Error.t()}

  @doc """
  Executes a query with an interface.
  """
  @spec execute(Clickhouse.Query.t()) :: any
  def execute(query) do
    query.__interface__.execute(query)
  end
end
