defmodule Clickhouse.Result do
  @moduledoc """
  The results of a Clickhouse query.
  """

  @type name :: binary()
  @type type :: atom() | nil
  @type columns :: [{name(), type()}, ...] | nil
  @type data :: [] | [list(), ...]
  @type t :: %__MODULE__{
          columns: columns(),
          data: data(),
          __raw__: binary(),
          __format__: Clickhouse.Format.t()
        }

  @derive {Inspect, only: [:data, :columns]}
  defstruct [:__raw__, :__format__, :columns, data: []]

  @spec new(binary(), Clickhouse.Format.t()) :: Clickhouse.Result.t()
  def new(raw, format) do
    %__MODULE__{__raw__: raw, __format__: format}
  end

  @spec put_columns(Clickhouse.Result.t(), columns()) :: Clickhouse.Result.t()
  def put_columns(result, columns) do
    %{result | columns: columns}
  end

  @spec put_data(Clickhouse.Result.t(), data()) :: Clickhouse.Result.t()
  def put_data(result, data) do
    %{result | data: data}
  end
end
